@@ -625,7 +625,7 @@ fill_tile (ChamplainMapSource *map_source,
               NULL);
           g_object_set (G_OBJECT (soup_session),
               "user-agent", "libchamplain/" CHAMPLAIN_VERSION_S,
-              "max-conns-per-host", 8, NULL); // This is the same has Firefox
+              "max-conns-per-host", 2, NULL); // This is as required by OSM
           g_object_add_weak_pointer (G_OBJECT (soup_session),
               (gpointer *) &soup_session);
         }
