@@ -55,8 +55,6 @@ enum
   PROP_ETAG
 };
 
-typedef struct _ChamplainTilePrivate ChamplainTilePrivate;
-
 struct _ChamplainTilePrivate {
   gint x;
   gint y;
@@ -69,6 +67,7 @@ struct _ChamplainTilePrivate {
   gchar *filename;
   ClutterActor *actor;
   ClutterActor *content_actor;
+  ClutterAnimation *animation;
 
   GTimeVal *modified_time;
   gchar* etag;
@@ -162,7 +161,15 @@ champlain_tile_set_property (GObject *object,
 static void
 champlain_tile_dispose (GObject *object)
 {
-  ChamplainTilePrivate *priv = GET_PRIVATE (object);
+  ChamplainTilePrivate *priv = CHAMPLAIN_TILE (object)->priv;
+
+  /* Force the animation to finish */
+  if (priv->animation != NULL)
+    {
+      clutter_animation_completed (priv->animation);
+      /* this will call the animation complete callback and free the context,
+       * the animation object will be unreffed too */
+    }
 
   if (priv->actor != NULL)
     {
@@ -182,7 +189,7 @@ champlain_tile_dispose (GObject *object)
 static void
 champlain_tile_finalize (GObject *object)
 {
-  ChamplainTilePrivate *priv = GET_PRIVATE (object);
+  ChamplainTilePrivate *priv = CHAMPLAIN_TILE (object)->priv;
 
   g_free (priv->uri);
   g_free (priv->filename);
@@ -374,6 +381,7 @@ static void
 champlain_tile_init (ChamplainTile *self)
 {
   ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  self->priv = priv;
 
   priv->state = CHAMPLAIN_STATE_INIT;
   priv->x = 0;
@@ -386,7 +394,10 @@ champlain_tile_init (ChamplainTile *self)
   priv->etag = NULL;
 
   priv->actor = g_object_ref (clutter_group_new ());
+  g_object_add_weak_pointer (G_OBJECT (priv->actor), (gpointer*)&priv->actor);
+
   priv->content_actor = NULL;
+  priv->animation = NULL;
 }
 
 /**
@@ -415,7 +426,7 @@ champlain_tile_get_x (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), 0);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   return priv->x;
 }
@@ -433,7 +444,7 @@ champlain_tile_get_y (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), 0);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   return priv->y;
 }
@@ -451,7 +462,7 @@ champlain_tile_get_zoom_level (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), 0);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   return priv->zoom_level;
 }
@@ -469,7 +480,7 @@ champlain_tile_get_size (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), 0);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   return priv->size;
 }
@@ -487,7 +498,7 @@ champlain_tile_get_state (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), CHAMPLAIN_STATE_NONE);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   return priv->state;
 }
@@ -505,7 +516,7 @@ champlain_tile_get_uri (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), NULL);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   return priv->uri;
 }
@@ -523,7 +534,7 @@ champlain_tile_get_filename (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), NULL);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   return priv->filename;
 }
@@ -542,9 +553,7 @@ champlain_tile_get_actor (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), NULL);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
-
-  return priv->actor;
+  return self->priv->actor;
 }
 
 /**
@@ -562,7 +571,7 @@ champlain_tile_set_x (ChamplainTile *self,
 {
   g_return_if_fail(CHAMPLAIN_TILE(self));
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   priv->x = x;
 
@@ -588,7 +597,7 @@ champlain_tile_set_y (ChamplainTile *self,
 {
   g_return_if_fail(CHAMPLAIN_TILE(self));
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   priv->y = y;
   if (priv->actor != NULL)
@@ -612,7 +621,7 @@ champlain_tile_set_zoom_level (ChamplainTile *self,
 {
   g_return_if_fail(CHAMPLAIN_TILE(self));
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   priv->zoom_level = zoom_level;
   g_object_notify (G_OBJECT (self), "zoom-level");
@@ -633,7 +642,7 @@ champlain_tile_set_size (ChamplainTile *self,
 {
   g_return_if_fail(CHAMPLAIN_TILE(self));
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   priv->size = size;
 
@@ -658,7 +667,7 @@ champlain_tile_set_state (ChamplainTile *self,
 {
   g_return_if_fail(CHAMPLAIN_TILE(self));
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   priv->state = state;
   g_object_notify (G_OBJECT (self), "state");
@@ -702,7 +711,7 @@ champlain_tile_set_uri (ChamplainTile *self,
   g_return_if_fail(CHAMPLAIN_TILE(self));
   g_return_if_fail(uri != NULL);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   g_free (priv->uri);
   priv->uri = g_strdup (uri);
@@ -720,12 +729,12 @@ champlain_tile_set_uri (ChamplainTile *self,
  */
 void
 champlain_tile_set_filename (ChamplainTile *self,
-                             const gchar *filename)
+    const gchar *filename)
 {
   g_return_if_fail(CHAMPLAIN_TILE(self));
   g_return_if_fail(filename != NULL);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   g_free (priv->filename);
   priv->filename = g_strdup (filename);
@@ -745,7 +754,7 @@ champlain_tile_get_modified_time (ChamplainTile *self)
 {
   g_return_val_if_fail (CHAMPLAIN_TILE(self), NULL);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   return priv->modified_time;
 }
@@ -766,7 +775,7 @@ champlain_tile_set_modified_time (ChamplainTile *self,
   g_return_if_fail (CHAMPLAIN_TILE(self));
   g_return_if_fail (time != NULL);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   g_free (priv->modified_time);
   priv->modified_time = g_memdup(time_, sizeof (GTimeVal));
@@ -784,7 +793,7 @@ gchar *
 champlain_tile_get_modified_time_string (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), NULL);
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   if (priv->modified_time == NULL)
     return NULL;
@@ -809,9 +818,8 @@ const gchar *
 champlain_tile_get_etag (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), "");
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
 
-  return priv->etag;
+  return self->priv->etag;
 }
 
 /**
@@ -829,7 +837,7 @@ champlain_tile_set_etag (ChamplainTile *self,
 {
   g_return_if_fail(CHAMPLAIN_TILE(self));
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
 
   g_free (priv->etag);
   priv->etag = g_strdup (etag);
@@ -846,11 +854,14 @@ fade_in_completed (ClutterAnimation *animation,
     gpointer data)
 {
   AnimationContext* ctx = (AnimationContext*) data;
-  ChamplainTilePrivate *priv = GET_PRIVATE (ctx->tile);
+  ChamplainTilePrivate *priv = ctx->tile->priv;
+
+  priv->animation = NULL;
 
   if (ctx->old_actor != NULL)
     {
-      clutter_container_remove (CLUTTER_CONTAINER (priv->actor), ctx->old_actor, NULL);
+      if (priv->actor != NULL)
+        clutter_container_remove (CLUTTER_CONTAINER (priv->actor), ctx->old_actor, NULL);
       g_object_unref (ctx->old_actor);
     }
 
@@ -877,9 +888,13 @@ champlain_tile_set_content (ChamplainTile *self,
   g_return_if_fail(CHAMPLAIN_TILE(self));
   g_return_if_fail(actor != NULL);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
+  ChamplainTilePrivate *priv = self->priv;
   ClutterActor *old_actor = NULL;
 
+  /* Don't start another animation if there's already one going on */
+  if (priv->animation != NULL)
+    clutter_animation_completed (priv->animation);
+
   if (priv->content_actor != NULL)
     {
       /* it sometimes happen that the priv->content_actor has been destroyed,
@@ -893,20 +908,20 @@ champlain_tile_set_content (ChamplainTile *self,
       g_object_unref (priv->content_actor);
     }
 
-  clutter_container_add (CLUTTER_CONTAINER (priv->actor), actor, NULL);
+  if (priv->actor != NULL)
+    clutter_container_add (CLUTTER_CONTAINER (priv->actor), actor, NULL);
 
-  if (fade_in == TRUE)
+  if (fade_in == TRUE && priv->actor !=  NULL)
     {
-      ClutterAnimation *animation;
       clutter_actor_set_opacity (actor, 0);
 
       AnimationContext *ctx = g_new0 (AnimationContext, 1);
       ctx->tile = g_object_ref (self);
-      animation = clutter_actor_animate (actor, CLUTTER_EASE_IN_CUBIC,
+      priv->animation = clutter_actor_animate (actor, CLUTTER_EASE_IN_CUBIC,
           500, "opacity", 255, NULL);
       ctx->old_actor = old_actor;
 
-      g_signal_connect (animation, "completed", G_CALLBACK (fade_in_completed), ctx);
+      g_signal_connect (priv->animation, "completed", G_CALLBACK (fade_in_completed), ctx);
     }
 
   priv->content_actor = g_object_ref (actor);
@@ -925,8 +940,6 @@ champlain_tile_get_content (ChamplainTile *self)
 {
   g_return_val_if_fail(CHAMPLAIN_TILE(self), NULL);
 
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
-
-  return priv->content_actor;
+  return self->priv->content_actor;
 }
 
